//******************************************************************************
//                                                                             *
//COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1994.   *
//                                                                             *
//DISCLAIMER OF WARRANTIES:                                                    *
//  The following [enclosed] code is sample code created by IBM                *
//  Corporation.  This sample code is not part of any standard IBM product     *
//  and is provided to you solely for the purpose of assisting you in the      *
//  development of your applications.  The code is provided "AS IS",           *
//  without warranty of any kind.  IBM shall not be liable for any damages     *
//  arising out of your use of the sample code, even if they have been         *
//  advised of the possibility of such damages.                                *
//                                                                             *
//******************************************************************************
//******************************************************************************
// File Name: store.hpp                                                        *
//                                                                             *
// Note: This file keeps the definiton of the store class.                     *
//                                                                             *
//******************************************************************************
#include <iostream.h>
#include <idsmcon.hpp>          // For Data Access Builder IDatastore Class
#include "inventov.hpp"         // For the generated Inventory object classes
#include "receivev.hpp"         // For the generated Receivedorder object classes
#include "prclistv.hpp"         // For the generated PriceList object classes

enum postflag {NOT_POSTED, POSTED};    // enum flag for posted or not yet posted
enum commitflag {ROLLBACK, COMMIT};    // enum flag for commit or rollback

class Store {
public:
   Store(IString databaseNameArg);     // Constructor expects the name of the database
   ~Store();                           // Destructor
   void generatePriceList();           // Generate a price list
   void postingInventory();            // update the inventory data with received order
   void displayItem();                 // display a price list item
   void transaction(commitflag commitflagArg); // commit or rollback

   void addReceivedOrder(              // Add a received order to the database
          short  receivedNumberArg,    // received order number
          IString prodNumberArg,       // product number
          long    receivedQuantityArg, // recieved quantity
          double  totalCostArg,        // total cost of the order
          IString receivedDateArg);    // date of the received order

   void addNewItem(                    // add a new item to the inventory table
          IString prodNumberArg,       // product number
          IString prodDescriptionArg,  // product description
          long    onHandQuantityArg,   // on hand quantity
          double  avarageCostArg,      // average cost per unit
          double  listingPriceArg);    // listing price per unit
private:
   IDatastore            dsm;                 // datastore manager
   Inventory             inventoryObj;        // inventory class object
   Receivedorder         receivedOrderObj;    // received order class object
   Receivedorder         *receivedOrderPtr;   // pointer to the order table
   ReceivedorderManager  receivedOrderMgr;    // received order manager
   PriceList             *priceListPtr;       // pointer to price list object
   PriceListManager      priceListMgr;        // price list object manager
};

