/*****************************************************************************
* TIME HANDLER SAMPLE PROGRAM - Version 6: Class Implementation(atimehdr.cpp)*
*                                                                            *
* COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1995. *
*                                                                            *
* DISCLAIMER OF WARRANTIES:                                                  *
*   The following [enclosed] code is sample code created by IBM              *
*   Corporation.  This sample code is not part of any standard IBM product   *
*   and is provided to you solely for the purpose of assisting you in the    *
*   development of your applications.  The code is provided "AS IS",         *
*   without warranty of any kind.  IBM shall not be liable for any damages   *
*   arising out of your use of the sample code, even if they have been       *
*   advised of the possibility of such damages.                              *
*****************************************************************************/
//NOTE: WE RECOMMEND USING A FIXED-SPACE FONT TO LOOK AT THE SOURCE.

#ifndef _IBASE_                         //Make sure ibase.hpp is included
  #include <ibase.hpp>                  //  since that is where IC_<environ>
#endif                                  //  is defined.
#ifdef IC_PM                            //For OS/2,                           V6
  #define TIMER_ID      1               //  Set the timer ID as a constant    V6
  #define INCL_WINTIMER                 //  Include the timer routines        V6
  #include <os2.h>                      //    from the OS/2 libraries         V6
  #include <ithread.hpp>                //  Include IThread class header      V6
#endif                                                                      //V6

#include <iwindow.hpp>                  //Include IWindow class header        V6

#ifdef IC_MOTIF                         //For AIX,                            V6
  #define WM_TIMER      0xF024          //  Create an event id for a timer    V6
  extern "C" {                          //  Include the X-Window header files V6
  #include <X11/Intrinsic.h>            //    as C externals                  V6
  }                                                                         //V6
  extern void _System                   //Forward declare for post function   V6
    postATimeHandlerEvent (IWindow *, XtIntervalId *);                      //V6
#endif                                                                      //V6

#include "atimehdr.hpp"                 //Include ATimeHandler class header   V6
const unsigned long                     //Set the timer interval to 1000      V6
  timeInterval = 1000;                  //  milliseconds (1 second)           V6

/**************************************************************************   V6
* ATimeHandler :: handleEventsFor                                         *   V6
*   Begin processing timer events for the window passed in by starting    *   V6
*     the appropriate system timer.                                       *   V6
**************************************************************************/ //V6
ATimeHandler                                                                //V6
  &ATimeHandler :: handleEventsFor( IWindow *window )                       //V6
{                                                                           //V6

#ifdef IC_MOTIF                                                             //V6
/*--------------------------- Start a Timer ------------------------------|   V6
| The X-Windows application add timeout routine is called by specifying,  |   V6
|   the application context which is obtained using the window handle,    |   V6
|   a time interval in milliseconds which is set to a constant 1000UL,    |   V6
|   an external routine that is called when the timer is up, and          |   V6
|   client_data which is a pointer to application specific data.          |   V6
| The timerId is a unique ID returned from the add timer routine that     |   V6
|   will be used when processing the expired timer.                       |   V6
| When the timer expires, the callback routine will be called with        |   V6
|   two parameters, the client_data and the timer id.                     |   V6
|------------------------------------------------------------------------*/ //V6
  timerId = XtAppAddTimeOut (                                               //V6
                   XtWidgetToApplicationContext ((Widget)window->handle()), //V6
                   timeInterval,                                            //V6
                   (XtTimerCallbackProc) postATimeHandlerEvent,             //V6
                   window);                                                 //V6
#endif                                                                      //V6

#ifdef IC_PM                                                                //V6
/*--------------------------- Start a Timer ------------------------------|   V6
| The Presentation Manager start timer routine is called by specifying    |   V6
|   the anchor block handle for the current thread,                       |   V6
|   the handle of the window passed in,                                   |   V6
|   a unique timer ID that is below the PM TID_USER_MAX constant, and     |   V6
|   a time interval in milliseconds which is set to a constant 1000UL.    |   V6
| Since this application uses only a single timer, the timer ID is        |   V6
|   specified as a constant.                                              |   V6
| When the timer expires, PM will post a WM_TIMER event to the window     |   V6
|   specified in parameter 2 of the WinStartTimer call.  IParameter1      |   V6
|   of the timer event will contain the timer ID.                         |   V6
|------------------------------------------------------------------------*/ //V6
  timerId = TIMER_ID;                                                       //V6
  WinStartTimer( IThread::current().anchorBlock(),                          //V6
                 window->handle(), timerId, timeInterval);                  //V6
#endif                                                                      //V6

/*------------------------- Start the Handler ----------------------------|   V6
| Start the handler by explicitly calling the overridden function.        |   V6
|------------------------------------------------------------------------*/ //V6
  Inherited::handleEventsFor(window);                                       //V6
  return (*this);                                                           //V6
} /* end ATimeHandler :: handleEventsFor(...) */                            //V6

#ifdef IC_MOTIF                                                             //V6
/**************************************************************************   V6
* external postATimeHandlerEvent - timer callback routine                 *   V6
*   Post a WM_TIMER event.  This is only necessary in MOTIF.              *   V6
*     OS/2 PM timer calls automatically post the WM_TIMER event.          *   V6
**************************************************************************/ //V6
  extern void _System                                                       //V6
    postATimeHandlerEvent (IWindow * window, XtIntervalId *timerUp)         //V6
{                                                                           //V6
/*-------------------------- Is Window Valid ? ---------------------------|   V6
| This test prevents an exception in IWindow in the case where a timer    |   V6
|   expires while the window is being closed.  If the window is not valid,|   V6
|   do nothing.                                                           |   V6
|------------------------------------------------------------------------*/ //V6
  if (window->isValid())                                                    //V6
{                                                                           //V6
/*--------------------------- Add Next Timer -----------------------------|   V6
| When an X-Windows timer is up, it does not restart automatically.       |   V6
|   Therefore, the callback routine starts another timer and registers    |   V6
|   itself as the callback routine.  The new timer ID is passed to the    |   V6
|   time handler in the timer event as parameter 2.                       |   V6
|------------------------------------------------------------------------*/ //V6
    IEventParameter2 newTimer = XtAppAddTimeOut (                           //V6
              XtWidgetToApplicationContext((Widget)window->handle()),       //V6
              timeInterval,                                                 //V6
              (XtTimerCallbackProc)postATimeHandlerEvent,                   //V6
              window);                                                      //V6
/*-------------------------- Post Timer Event ----------------------------|   V6
| The timer event is created by posting an event to the window passed     |   V6
|   in as client_data to the callback routine.  The ID of the timer that  |   V6
|   expired is passed to the time handler as IParameter1 of the event.    |   V6
|------------------------------------------------------------------------*/ //V6
    window->postEvent (WM_TIMER, IEventParameter1(*timerUp), newTimer);     //V6
  }                                                                         //V6
} /* end extern void _System postATimeHandlerEvent(...) */                  //V6
#endif                                                                      //V6

/**************************************************************************   V6
* ATimeHandler :: dispatchHandlerEvent                                    *   V6
*   Call tick when one of our timer events are dispatched.                *   V6
**************************************************************************/ //V6
IBase::Boolean                                                              //V6
  ATimeHandler :: dispatchHandlerEvent(IEvent& event)                       //V6
{                                                                           //V6
  Boolean eventProcessed(false);        //Assume event will not be proccessed V6
/*--------------------------- Test the Event -----------------------------|   V6
| This event must be a timer event and parameter 1 must contain the       |   V6
|   ID of the timer that was started by this handler.                     |   V6
|------------------------------------------------------------------------*/ //V6
  if ((event.eventId() == WM_TIMER) && (event.parameter1() == timerId))     //V6
  {                                                                         //V6
#ifdef IC_MOTIF                                                             //V6
/*--------------------------- Save New Timer ID --------------------------|   V6
| Since the timer ID is not constant in MOTIF, the new timer id, which    |   V6
|   is passed as parameter 2 of the event, must be saved.                 |   V6
|------------------------------------------------------------------------*/ //V6
   timerId = event.parameter2();                                            //V6
#endif                                                                      //V6
/*------------------------- Process Timer Event --------------------------|   V6
| Process the timer event by calling the virtual function tick, which     |   V6
|   should be overridden by an inheriting class.  The returned boolean    |   V6
|   value determines whether the event was actually processed by tick.    |   V6
|   The default ATimeHandler::tick function does not process the event.   |   V6
|------------------------------------------------------------------------*/ //V6
    eventProcessed = tick(event);                                           //V6
  }                                                                         //V6
  return (eventProcessed);                                                  //V6
} /* end ATimeHandler :: dispatchHandlerEvent(...) */                       //V6

/**************************************************************************   V6
* ATimeHandler :: tick                                                    *   V6
*   Default tick handler that should be overridden by inheriting classes  *   V6
**************************************************************************/ //V6
IBase::Boolean                                                              //V6
  ATimeHandler :: tick(IEvent& event)                                       //V6
{                                                                           //V6
  return (false);                       //The timer event is not processed  //V6
} /* end ATimeHandler :: tick(...) */                                       //V6

/**************************************************************************   V6
* ATimeHandler :: stopHandlingEventsFor                                   *   V6
*   Stop processing timer events for the window passed in.                *   V6
**************************************************************************/ //V6
ATimeHandler                                                                //V6
  &ATimeHandler :: stopHandlingEventsFor( IWindow *window )                 //V6
{                                                                           //V6

#ifdef IC_MOTIF                                                             //V6
/*--------------------------- Stop the Timer -----------------------------|   V6
| The X-Windows remove timeout routine is called to stop the outstanding  |   V6
|   timer which is identified by the ID stored in timerId.                |   V6
|------------------------------------------------------------------------*/ //V6
  XtRemoveTimeOut (timerId);                                                //V6
  timerId = 0;                                                              //V6
#endif                                                                      //V6

#ifdef IC_PM                                                                //V6
/*--------------------------- Stop the Timer -----------------------------|   V6
| The Presentation Manager stop timer routine is called to stop the       |   V6
|   outstanding timer which is identified by,                             |   V6
|     the anchor block handle of the current thread,                      |   V6
|     the handle of the window passed in, and                             |   V6
|     the ID stored in timerId.                                           |   V6
|------------------------------------------------------------------------*/ //V6
  WinStopTimer( IThread::current().anchorBlock(),                           //V6
                window->handle(), timerId);                                 //V6
#endif                                                                      //V6

/*-------------------------- Stop the Handler ----------------------------|   V6
| Stop the handler by explicitly calling the overridden function.  Its    |   V6
|   returned value is used to determine the success of this routine.      |   V6
|------------------------------------------------------------------------*/ //V6
  Inherited::stopHandlingEventsFor(window);                                 //V6
  return (*this);                                                           //V6
} /* end ATimeHandler :: stopHandlingEventsFor(...) */                      //V6

