##########################################################
# Makefile (cppov13.MAK) to build a C++ DLL for
# all Calculator non-visual parts built by Alpha Omega.
##########################################################
#
# COPYRIGHT: Copyright (C) International Business Machines Corp., 1992,1993.
#
##########################################################


##########################################################
# Define variables for makefile
##########################################################
GCPPFLAGS=/c /Ge- /Ti+ /Gd- /Gm+ /Gn+ /Ftcppov13 /Tdp -DIC_TRACE_DEVELOP
#GCPPFLAGS=/c /Ge- /Ti+ /Gd- /Gm+ /Gn+ /Tdp -DIC_TRACE_DEVELOP
#GCPPFLAGS=/c /Ge- /Ti+ /Gd- /Gm+ /Gn+ /Ftcppov13 /Tdp \
#         /I .;\\tf4serv2\csbuild\idriver2\ibmclass
#GCPPFLAGS=/c /Ge- /Ti+
#GCPPFLAGS=-c -O+ -Ge-
#GCPPFLAGS=-Fd -c -Gm+ -Ti+ -Ge-
#GCPPFLAGS=-Fd -c -Gm+ -O+
##########################################################
# Definitions for GCPPFLAGS (compile options):
##########################################################
# /c   = perform compile only
# /Ge- = create a DLL (/Ge+ to create an EXE)
# /Ti+ = generate debugger information
# /Gd- = statically link runtime functions (why??)
# /Gm+ = multi-thread capability
# /Gn+ = not providing linker info about default libraries
#        (in object).  All libraries must be explicitly
#        specified at link time
# /Ft  = generate files for template resolution and put
#        them in the specified directory
# /Tdp = consider all files to be C++ files
# /I   = control which paths are searched when the
#        compiler looks for user #include files.  These 
#        paths are searched before those given in the 
#        INCLUDE environment variable.  The format is:
#             I  path[;path][;path]...
##########################################################

all:          main

main:         cppov13.dll

##########################################################
# List of dependency files for the DLL.
# NOTE:  the compilation ordering is determined by the
#        dependency ordering.
##########################################################
cppov13.dll:    cppov13.def \
                icalcpu.obj \
                icalcop.obj \
                icalcfnc.obj \
                ikeypad.obj \
                icalcac.obj \
                icalcsc.obj \
                icalcmc.obj \
                icalcdc.obj \
                icalcclr.obj \
                icalceql.obj \
                icalctly.obj \
                iclrdsp.obj \
                imykybrd.obj
##########################################################
# When any of the DLL dependency files change, invoke the
# linker to rebuild the DLL.
##########################################################
# /Tdp = consider all files to be C++ files
# /Gm+ = multi-thread capability
# /Gn+ = not providing linker info about default libraries
#        (in object).  All libraries must be explicitly
#        specified at link time
# /Ft  = generate files for template resolution in the
#        specified directory under the current directory
# /B   = indicates passing linker options
# /NOE = 
# /NOI = 
# /MAP = 
#
# NOTE:  Dave said that the template code doesn't get compiled until
#        the link, so need to include the /Gm+ and /Gn+ options on the
#        link as well as the compile (otherwise, got link errors).
##########################################################
                icc /Tdp /Gm+ /Gn+ /Ftcppov13 /B"/NOE /NOI /MAP /DEBUG" \
#               icc /Tdp /Gm+ /Gn+ /B"/NOE /NOI /MAP /DEBUG" \
#               icc /Tdp /Ftcppov13 /DIC_TRACE_DEVELOP /B"/NOE /NOI /MAP /DEBUG" \
#               icc /tdp /B"/NOE /NOI /MAP" \
#               icc /Tdp /B" /PM:PM /MAP" \
##########################################################
# List of .OBJ files
##########################################################
                icalcop.obj \
                icalcfnc.obj \
                icalcpu.obj \
                ikeypad.obj \
                icalcac.obj \
                icalcsc.obj \
                icalcmc.obj \
                icalcdc.obj \
                icalcclr.obj \
                icalceql.obj \
                icalctly.obj \
                iclrdsp.obj \
                imykybrd.obj \
##########################################################
# Name of the DLL being built and the MAP file to be 
# created by the linker.
##########################################################
                /Fecppov13.dll /Fmcppov13.map \
##########################################################
# List of standard or import libraries (not DLLs) to search
##########################################################
# dde4mbs  = compiler lib      --> dde4mbsi.lib 6/21/94
# dde4muii = ibmclass lib
# dde4cci  = collection lib
# os2386   = for DOS and PM resolutions
# iavlbase,iavlsamp,iavlui --> iavalon.lib 6/21/94
# conversi.lib --> removed 9/12/94
##########################################################
                cppom30i.lib cppooc3i.lib cppooc3i.lib os2386.lib \
                cppoov3i.lib  \
##########################################################
# The module definition file (.DEF) for the DLL
##########################################################
                cppov13.def


##########################################################
# Generate an import library (.LIB) to allow DLLs to be 
# linked with applications.  The DEF file is used as input.
##########################################################
                implib cppov13i.lib cppov13.dll


##########################################################
# Statements to compile each non-visual C++ file for the Calc
##########################################################
icalcpu.obj:     icalcpu.cpp  icalcpu.hpp
                 icc $(GCPPFLAGS) icalcpu.cpp

icalcop.obj:     icalcop.cpp  icalcop.hpp
                 icc $(GCPPFLAGS) icalcop.cpp

icalcac.obj:     icalcac.cpp  icalcac.hpp icalcop.hpp
                 icc $(GCPPFLAGS) icalcac.cpp

icalcsc.obj:     icalcsc.cpp  icalcsc.hpp icalcop.hpp
                 icc $(GCPPFLAGS) icalcsc.cpp

icalcmc.obj:     icalcmc.cpp  icalcmc.hpp icalcop.hpp
                 icc $(GCPPFLAGS) icalcmc.cpp

icalcdc.obj:     icalcdc.cpp  icalcdc.hpp icalcop.hpp
                 icc $(GCPPFLAGS) icalcdc.cpp

icalcfnc.obj:    icalcfnc.cpp  icalcfnc.hpp
                 icc $(GCPPFLAGS) icalcfnc.cpp

icalcclr.obj:    icalcclr.cpp  icalcclr.hpp icalcfnc.hpp icalcpu.hpp
                 icc $(GCPPFLAGS) icalcclr.cpp

icalceql.obj:    icalceql.cpp  icalceql.hpp icalcfnc.hpp icalcpu.hpp
                 icc $(GCPPFLAGS) icalceql.cpp

icalctly.obj:    icalctly.cpp  icalctly.hpp
                 icc $(GCPPFLAGS) icalctly.cpp

ikeypad.obj:     ikeypad.cpp  ikeypad.hpp
                 icc $(GCPPFLAGS) ikeypad.cpp

iclrdsp.obj: iclrdsp.cpp iclrdsp.hpp
                 icc $(GCPPFLAGS) iclrdsp.cpp

imykybrd.obj: imykybrd.cpp imykybrd.hpp
                 icc $(GCPPFLAGS) imykybrd.cpp

