######################################
# Make file (cppov23.MAK) to build a C++ DLL for the
# sample Police Rap Sheet application.
##########################################################

##############################################################################
## COPYRIGHT: Copyright (C) International Business Machines Corp., 1994,1995 *
##                                                                           *
## DISCLAIMER OF WARRANTIES:                                                 *
##   The following [enclosed] code is sample code created by IBM             *
##   Corporation.  This sample code is not part of any standard IBM product  *
##   and is provided to you solely for the purpose of assisting you in the   *
##   development of your applications.  The code is provided "AS IS",        *
##   without warranty of any kind.  IBM shall not be liable for any damages  *
##   arising out of your use of the sample code, even if they have been      *
##   advised of the possibility of such damages.                             *
##############################################################################
##NOTE: WE RECOMMEND USING A FIXED-SPACE FONT TO LOOK AT THE SOURCE.
##


##########################################################
# Define variables for makefile
##########################################################
GCPPFLAGS=/Ge- /Gd+ /Gm+ /Gn+ /Td /Ftcppov23 /I. -DIC_TRACE_DEVELOP

##########################################################
# Definitions for GCPPFLAGS (compile options):
##########################################################
# /c   = perform compile only
# /Ge- = create a DLL (/Ge+ to create an EXE)
# /Ti+ = generate debugger information
# /Gd+ = dynamically link runtime functions
# /Gm+ = multi-thread capability
# /Gn+ = not providing linker info about default libraries
#        (in object).  All libraries must be explicitly
#        specified at link time
# /Tdp = consider all files to be C++ files
# /Ft  = generate files for template resolution and put
#        them in the specified directory
# /I   = control which paths are searched when the
#        compiler looks for user #include files.  These 
#        paths are searched before those given in the 
#        INCLUDE environment variable.  The format is:
#             I  path[;path][;path]...
##########################################################

all:          main

main:         cppovo23.dll

##########################################################
# List of dependency files for the DLL.
# NOTE:  the compilation order is determined by the
#        dependency order.
##########################################################
cppovo23.dll: cppov23.def \
              querydb.obj \
              dbquery.obj \
              irsltrec.obj \
              iaddrrec.obj \
              ialsrec.obj \
              iarstrec.obj \
              ilsrec.obj \
              isusrec.obj \
              ialias.obj \
              iarrest.obj \
              ilstseen.obj \
              iperson.obj \
              isuspect.obj \
              ialsrmgr.obj \
              iarsrmgr.obj \
              iadrrmgr.obj \
              ilsrmgr.obj \
              isusrmgr.obj 
##########################################################
# When any of the DLL dependency files change, invoke the
# linker to rebuild the DLL.
##########################################################
# NOTE:  Dave said that the template code doesn't get compiled until
#        the link, so need to include the /Gm+ and /Gn+ options on the
#        link as well as the compile (otherwise, got link errors).
##########################################################
              icc $(GCPPFLAGS) /Tdp /B"/NOE /NOI /MAP /DEBUG /NOP" \
##########################################################
# List of .OBJ files
##########################################################
                querydb.obj \
                dbquery.obj \
                irsltrec.obj \
                iaddrrec.obj \
                ialsrec.obj \
                iarstrec.obj \
                ilsrec.obj \
                isusrec.obj \
                ialias.obj \
                iarrest.obj \
                ilstseen.obj \
                iperson.obj \
                isuspect.obj \
                ialsrmgr.obj \
                iarsrmgr.obj \
                iadrrmgr.obj \
                ilsrmgr.obj \
                isusrmgr.obj \
##########################################################
# Name of the DLL being built and the MAP file to be 
# created by the linker.
##########################################################
                /Fecppov23.dll /Fmcppov23.map \
##########################################################
# List of standard or import libraries (not DLLs) to search
##########################################################
# os2386   = for DOS and PM resolutions
# cppom30i = compiler lib    
# cppooc3i = open class lib
# cppov03i = sample lib
##########################################################
                os2386.lib cppom30i.lib cppooc3i.lib cppov03i.lib sql_dyn.lib \
##########################################################
# The module definition file (.DEF) for the DLL
##########################################################
                cppov23.def

##########################################################
# Generate an import library (.LIB) to allow DLLs to be 
# linked with applications.  The DEF file is used as input.
##########################################################
              implib cppov23i.lib cppov23.dll

##########################################################
# Bind the SQL code to the RAP database.
##########################################################
bind:         dbquery.bnd



##########################################################
# Statements to pre-compile SQL code and bind it to the database
##########################################################
dbquery.c:    dbquery.sqc dbquery.h rap.h
              startdbm
              sqlprep dbquery.sqc RAP /b
              sqlbind dbquery.bnd RAP

##########################################################
# Statements to compile each non-visual C/C++ file for the Rap
##########################################################
querydb.obj:  querydb.cpp  querydb.hpp irsltrec.hpp isusrec.hpp dbquery.h rap.h
              icc -c $(GCPPFLAGS) querydb.cpp

dbquery.obj:  dbquery.c  rap.h dbquery.h
              icc -c $(GCPPFLAGS) dbquery.c

irsltrec.obj: irsltrec.cpp  irsltrec.hpp
              icc -c $(GCPPFLAGS) irsltrec.cpp

isusrec.obj:  isusrec.cpp  isusrec.hpp
              icc -c $(GCPPFLAGS) isusrec.cpp

iaddrrec.obj: iaddrrec.cpp  iaddrrec.hpp
              icc -c $(GCPPFLAGS) iaddrrec.cpp

ialsrec.obj:  ialsrec.cpp  ialsrec.hpp
              icc -c $(GCPPFLAGS) ialsrec.cpp

iarstrec.obj: iarstrec.cpp  iarstrec.hpp
              icc -c $(GCPPFLAGS) iarstrec.cpp

ilsrec.obj:   ilsrec.cpp  ilsrec.hpp
              icc -c $(GCPPFLAGS) ilsrec.cpp

ialias.obj:   ialias.cpp {$(INCLUDE)}ialias.hpp
              icc -c $(GCPPFLAGS) ialias.cpp

iarrest.obj:  iarrest.cpp {$(INCLUDE)}iarrest.hpp {$(INCLUDE)}rap.h
              icc -c $(GCPPFLAGS) iarrest.cpp

ilstseen.obj: ilstseen.cpp {$(INCLUDE)}ilstseen.hpp {$(INCLUDE)}iadd.hpp
              icc -c $(GCPPFLAGS) ilstseen.cpp

iperson.obj:  iperson.cpp {$(INCLUDE)}iperson.hpp {$(INCLUDE)}iadd.hpp \
              {$(INCLUDE)}ialias.hpp {$(INCLUDE)}ivseq.h {$(INCLUDE)}rap.h
              icc -c $(GCPPFLAGS) iperson.cpp

isuspect.obj: isuspect.cpp {$(INCLUDE)}isuspect.hpp {$(INCLUDE)}iarrest.hpp \
              {$(INCLUDE)}ilstseen.hpp {$(INCLUDE)}ivseq.h
              icc -c $(GCPPFLAGS) isuspect.cpp

ialsrmgr.obj: ialsrmgr.cpp {$(INCLUDE)}ialsrmgr.hpp {$(INCLUDE)}irsltrec.hpp \
              {$(INCLUDE)}ialsrec.hpp {$(INCLUDE)}ialias.hpp {$(INCLUDE)}querydb.hpp \
              {$(INCLUDE)}ivseq.h
              icc -c $(GCPPFLAGS) ialsrmgr.cpp

iarsrmgr.obj: iarsrmgr.cpp {$(INCLUDE)}iarsrmgr.hpp {$(INCLUDE)}irsltrec.hpp \
              {$(INCLUDE)}iarstrec.hpp {$(INCLUDE)}iarrest.hpp {$(INCLUDE)}querydb.hpp \
              {$(INCLUDE)}ivseq.h
              icc -c $(GCPPFLAGS) iarsrmgr.cpp

iadrrmgr.obj: iadrrmgr.cpp {$(INCLUDE)}iadrrmgr.hpp {$(INCLUDE)}irsltrec.hpp \
              {$(INCLUDE)}iaddrrec.hpp {$(INCLUDE)}iadd.hpp {$(INCLUDE)}querydb.hpp \
              {$(INCLUDE)}ivseq.h
              icc -c $(GCPPFLAGS) iadrrmgr.cpp

ilsrmgr.obj:  ilsrmgr.cpp {$(INCLUDE)}ilsrmgr.hpp {$(INCLUDE)}irsltrec.hpp \
              {$(INCLUDE)}ilsrec.hpp {$(INCLUDE)}ilstseen.hpp {$(INCLUDE)}querydb.hpp \
              {$(INCLUDE)}ivseq.h
              icc -c $(GCPPFLAGS) ilsrmgr.cpp

isusrmgr.obj: isusrmgr.cpp  {$(INCLUDE)}isusrmgr.hpp {$(INCLUDE)}irsltrec.hpp \
              {$(INCLUDE)}isusrec.hpp {$(INCLUDE)}isuspect.hpp {$(INCLUDE)}querydb.hpp \
              {$(INCLUDE)}iadd.hpp {$(INCLUDE)}ivseq.h
              icc -c $(GCPPFLAGS) isusrmgr.cpp
