#ifndef _ILSRMGR_
#define _ILSRMGR_
//****************************************************************************
// ILastSeenClientMgr Class - C++ Header File (ilsrmgr.hpp)                  *
//                                                                           *
// COPYRIGHT: Copyright (C) International Business Machines Corp., 1994,1995 *
//                                                                           *
// DISCLAIMER OF WARRANTIES:                                                 *
//   The following [enclosed] code is sample code created by IBM             *
//   Corporation.  This sample code is not part of any standard IBM product  *
//   and is provided to you solely for the purpose of assisting you in the   *
//   development of your applications.  The code is provided "AS IS",        *
//   without warranty of any kind.  IBM shall not be liable for any damages  *
//   arising out of your use of the sample code, even if they have been      *
//   advised of the possibility of such damages.                             *
//****************************************************************************
//NOTE: WE RECOMMEND USING A FIXED-SPACE FONT TO LOOK AT THE SOURCE.
//

#include <istdntfy.hpp>               //Parent of ILastSeenClientMgr
#include <istring.hpp>                //IString header
#include <ivseq.h>                    //IVSequence header
#include <ilstseen.hpp>               //ILastSeen header

//Forward declarations of other classes

//**************************************************************************
// Class:   ILastSeenClientMgr
//
// Purpose: Describes ILastSeenClientMgr
//
//**************************************************************************
class _Export ILastSeenClientMgr : public IStandardNotifier
{

public:
/*---------------------  CONSTRUCTORS/DESTRUCTORS  ----------------
 *-----------------------------------------------------------------*/
ILastSeenClientMgr ();
virtual  ~ILastSeenClientMgr ();


/*---------------------  NOTIFICATION EVENTS  ----------------------
 | These INotificationId strings are used for all notifications that IWindow
 | provides to its observers:
 |     -parseResultId      Notification identifier provided to
 |                         observers when the parsing of dResultBuffer
 |                         has completed.
 |     -resultBufferId     Notification identifier provided to
 |                         observers when the resultBuffer attribute
 |                         changes.
 |     -resultListId       Notification identifier provided to
 |                         observers when the resultList attribute
 |                         changes.
 |     -resultObjectId     Notification identifier provided to
 |                         observers when the resultObject attribute
 |                         changes.
 |     -resultWrapperId    Notification identifier provided to
 |                         observers when the resultWrapper attribute
 |                         changes.
 *-----------------------------------------------------------------*/
static INotificationId parsedResultId;
static INotificationId requestBufferId;
static INotificationId resultBufferId;
static INotificationId resultListId;
//temporarily added resultListWrapperId
static INotificationId resultListWrapperId;
static INotificationId resultObjectId;
static INotificationId resultWrapperId;
static INotificationId sameRequestId;
static INotificationId manyObjectsFoundId;
static INotificationId noObjectsFoundId;
static INotificationId oneObjectFoundId;


/*---------------------  ATTRIBUTES  -------------------------------
 | The following members are for accessing the attributes of this class:
 |     -resultBuffer       Returns the resultBuffer attribute.
 |     -resultList         Returns resultList attribute, which
 |                         represents the list of lastSeen objects
 |                         that were created via parsing the
 |                         resultBuffer attribute.
 |     -resultObject       Returns the resultObject attribute, which
 |                         represents the first lastSeen object parsed
 |                         from the resultBuffer attribute.
 |     -resultWrapper      Returns a pointer to the resultObject
 |                         attribute.
 |     -setResultBuffer    Sets the resultBuffer attribute.
 |                         NOTE: the IString passed in should map to
 |                               an IResultRecord, where the resultData
 |                               of the IResultRecord represents zero
 |                               or more lastSeen records (ILastSeenRecord).
 |                               numResults of the IResultRecord must
 |                               indicate how many lastSeen records
 |                               resultData contains.
 *-----------------------------------------------------------------*/
virtual IString
  requestBuffer () const;
virtual IString
  resultBuffer () const;
virtual IVSequence <ILastSeen *>
  resultList () const;
virtual IVSequence <ILastSeen *> *
  resultListWrapper () const;
virtual ILastSeen
  resultObject () const;
virtual ILastSeen *
  resultWrapper () const;
virtual ILastSeenClientMgr&
  setRequestBuffer (const IString & iRequestBuffer);
virtual ILastSeenClientMgr&
  setResultBuffer (const IString & iResultBuffer);


/*---------------------  ACTIONS   --------------------------------- 
 | allByBookNumber()  Retrieves from the database a list of locations
 |                    where a given suspect has been sited.
 | processResult()    Creates a list of adddress objects by parsing
 |                    information contained in the resultBuffer
 |                    attribute.
 *-----------------------------------------------------------------*/
virtual IString 
   allByBookNumber (unsigned long aBookNumber);
virtual ILastSeenClientMgr &
   processResult();


/*---------------------  OPERATORS   -------------------------------
 *-----------------------------------------------------------------*/


protected:
/*---------------------  ENUMERATED TYPES  ------------------------
 *-----------------------------------------------------------------*/

/*---------------------  PROTECTED SERVICES  ----------------------
| These operations or services provided by this class:
-------------------------------------------------------------------*/


private:
/*---------------------  PRIVATE MEMBER DATA  ---------------------
 *-----------------------------------------------------------------*/
  IString dRequestBuffer;
  IString dResultBuffer;
  ILastSeen *dResultWrapper;
  IVSequence <ILastSeen *> *dResultListWrapper;

/*---------------------  PRIVATE SERVICES  -------------------------
| These operations or services provided by this class:
|    addResult()              adds an lastSeen object's pointer to
|                             the result list.
|    setResultWrapper()       sets the dResultList member data.
-------------------------------------------------------------------*/
ILastSeenClientMgr & ILastSeenClientMgr ::
  addResult(ILastSeen * iLastSeen);
ILastSeenClientMgr & ILastSeenClientMgr ::
  addResult(ILastSeen & iLastSeen);

ILastSeenClientMgr&
  setResultWrapper (ILastSeen * iResultWrapper);
ILastSeenClientMgr&
  setResultWrapper (const ILastSeen & iResult);

};

#endif
