#ifndef _ILASTSEEN_
#define _ILASTSEEN_
//****************************************************************************
// ILastSeen Class - C++ Header File (ilstseen.hpp)                          *
//                                                                           *
// COPYRIGHT: Copyright (C) International Business Machines Corp., 1994,1995 *
//                                                                           *
// DISCLAIMER OF WARRANTIES:                                                 *
//   The following [enclosed] code is sample code created by IBM             *
//   Corporation.  This sample code is not part of any standard IBM product  *
//   and is provided to you solely for the purpose of assisting you in the   *
//   development of your applications.  The code is provided "AS IS",        *
//   without warranty of any kind.  IBM shall not be liable for any damages  *
//   arising out of your use of the sample code, even if they have been      *
//   advised of the possibility of such damages.                             *
//****************************************************************************
//NOTE: WE RECOMMEND USING A FIXED-SPACE FONT TO LOOK AT THE SOURCE.
//

#include <istdntfy.hpp>             //Parent of ILastSeen
#include <istring.hpp>
#include <iadd.hpp>

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

//**************************************************************************
// Class:   ILastSeen
//
// Purpose: Describes ILastSeen Class.
//   This class defines ILastSeen objects for use in applications such as the
//      Police Rap Sheet.
//   These objects hold and manage all the data relevant to the sighting
//      of a suspect by a witness.
//
//**************************************************************************
class _Export ILastSeen : public IStandardNotifier
{

public:


/*---------------------  CONSTRUCTORS/DESTRUCTORS  ----------------
 *-----------------------------------------------------------------*/
ILastSeen();
virtual  ~ILastSeen();


/*---------------------  NOTIFICATION EVENTS  ----------------------
 | These INotificationId strings are used for all notifications that 
 | this class provides to its observers:
 |     -locationID         Notification identifier provided to
 |                         observers when the location attribute changes.
 |     -witnessID          Notification identifier provided to
 |                         observers when the witness  attribute changes.
 |     -dateID             Notification identifier provided to
 |                         observers when the location attribute changes.
 |     -commentID          Notification identifier provided to
 |                         observers when the comment  attribute changes.
 *-----------------------------------------------------------------*/
static INotificationId locationID;     
static INotificationId witnessID ;     
static INotificationId dateID    ;     
static INotificationId commentID ;     
static INotificationId addressId;


/*---------------------  ATTRIBUTES  -------------------------------
 | The following members are for accessing the attributes of this class:
 |     -location           Returns the location attribute.
 |     -setLocation        Sets the location  attribute.
 |     -witness            Returns the witness  attribute.
 |     -setWitness         Sets the witness   attribute.
 |     -dateLastSeen       Returns the date     attribute.
 |     -setDateLastSeen    Sets the date      attribute.
 |     -comment            Returns the comment  attribute.
 |     -setComment         Sets the comment   attribute.
 *-----------------------------------------------------------------*/
virtual IString location() const;
virtual ILastSeen & setLocation(const IString & aLocation);
virtual IString witness() const;
virtual ILastSeen & setWitness(const IString & aWitness);
virtual IString dateLastSeen() const;
virtual ILastSeen & setDateLastSeen(const IString & aDate);
virtual IString comment() const;
virtual ILastSeen & setComment(const IString & aComment);
virtual IAddress * address() const;
virtual ILastSeen & setAddress(IAddress * aAddress);

/*---------------------  ACTIONS   --------------------------------- 
 | asString    Formatted string for listbox and debugging output.
 *-----------------------------------------------------------------*/
virtual IString  
   asString() const;


/*---------------------  OPERATORS   -------------------------------
 *-----------------------------------------------------------------*/
Boolean
   operator ==(const ILastSeen & value) const;
   operator ==(const ILastSeen * value) const;
   operator !=(const ILastSeen & value) const;
   operator !=(const ILastSeen * value) const;

protected:

private:
/*---------------------  PRIVATE MEMBER DATA  ---------------------
 *-----------------------------------------------------------------*/
   IString        dLocation;
   IString        dWitness;
   IString        dDate;
   IString        dComment;
   IAddress       *iAddress;

};   //ILastSeen

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif
